/*
*This code combines the DHT sensor as well as a binary switch used as a bed occupancy sensor.
*There is a delay on the frequency that the DHT sensor reports back to Vera and can be
*configured with the DHTDelay variable.
*/

//#include <Sleep_n0m1.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <RF24.h>
#include <Sensor.h>  
#include <DHT.h>  
#include <Bounce2.h>

//Temp/Humidity Sensor
#define CHILD_ID_HUM 0 
#define CHILD_ID_TEMP 1
#define HUMIDITY_SENSOR_DIGITAL_PIN 4 //Connect DHT sensor to this pin

//Occupancy Sensor
#define DIGITAL_INPUT_SENSOR 3   // The digital input you attached your motion sensor.  (Only 2 and 3 generates interrupt!)
//#define INTERRUPT DIGITAL_INPUT_SENSOR-2 // Usually the interrupt = pin -2 (on uno/nano anyway)
#define CHILD_ID_OCCUPANCY 2   // Id of the sensor child

//unsigned long SLEEP_TIME = 30; // Sleep time between reads (in seconds)

Sensor gw;
DHT dht;
//Sleep sleep;
float lastTemp;
float lastHum;
boolean metric = true; 

Bounce debouncer = Bounce(); 
int oldValue=-1;

unsigned long previousMillis = 0; // last time update. See http://stackoverflow.com/questions/10773425/performing-a-function-after-x-time for more details on Millis delay
long DHTDelay = 120000; // interval at which to do something (milliseconds)


void setup()  
{ 
  gw.begin();
  
  //DHT code
  dht.setup(HUMIDITY_SENSOR_DIGITAL_PIN); 
  
  // sets the bed occupancy sensor digital pin as input
  pinMode(DIGITAL_INPUT_SENSOR,INPUT);      
  // Activate internal pull-up
  digitalWrite(DIGITAL_INPUT_SENSOR,HIGH);
    
  
  // After setting up the button, setup debouncer
  debouncer.attach(DIGITAL_INPUT_SENSOR);
  debouncer.interval(5);
  
  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo("Humidity+Occupancy", "1.0");

  // Register all sensors to gw (they will be created as child devices)
  gw.sendSensorPresentation(CHILD_ID_HUM, S_HUM);
  gw.sendSensorPresentation(CHILD_ID_TEMP, S_TEMP);
  gw.sendSensorPresentation(CHILD_ID_OCCUPANCY, S_MOTION);
  
  metric = gw.isMetricSystem();
}

void loop()      
{  
  //Delay temperature and humidity for the length of time of DHTDelay without stopping the whole program
  unsigned long currentMillis = millis();
  if(currentMillis - previousMillis > DHTDelay) {
    previousMillis = currentMillis;  
      
    delay(dht.getMinimumSamplingPeriod());
    
    float temperature = dht.getTemperature();
    if (isnan(temperature)) {
        Serial.println("Failed reading temperature from DHT");
    } else if (temperature != lastTemp) {
      lastTemp = temperature;
      if (!metric) {
        temperature = dht.toFahrenheit(temperature);
      }
      gw.sendVariable(CHILD_ID_TEMP, V_TEMP, temperature, 1);
        Serial.print("T: ");
        Serial.println(temperature);
    }
    
    float humidity = dht.getHumidity();
    if (isnan(humidity)) {
        Serial.println("Failed reading humidity from DHT");
    } else if (humidity != lastHum) {
        lastHum = humidity;
        gw.sendVariable(CHILD_ID_HUM, V_HUM, humidity, 1);
        Serial.print("H: ");
        Serial.println(humidity);
    }
  }
  
  //Code for binary switch (occupancy sensor)
  debouncer.update();
  // Get the update value
  int value = debouncer.read();
  
  
  
  if (value != oldValue) {
     // Send in the new value
     gw.sendVariable(DIGITAL_INPUT_SENSOR, V_TRIPPED, value==0 ? "1" : "0");  // Change to V_LIGHT if you use S_LIGHT in presentation above
     oldValue = value;
     Serial.println(value);
  }

  // Power down the radio.  Note that the radio will get powered back up
  // on the next write() call.
  //delay(1000); //delay to allow serial to fully print before sleep
//  gw.powerDown();
//  sleep.pwrDownMode(); //set sleep mode
//  sleep.sleepDelay(SLEEP_TIME * 1000); //sleep for: sleepTime
//  sleep.sleepInterrupt(INTERRUPT,CHANGE); 
}



